@@ -3,7 +3,7 @@
 # Copyright (c) 1994-1999 Malcolm Beattie. All rights reserved.
 # Copyright (c) 2003 Enache Adrian. All rights reserved.
 # Copyright (c) 2008-2011 Reini Urban <rurban@cpan.org>. All rights reserved.
-# Copyright (c) 2011-2014 cPanel Inc. All rights reserved.
+# Copyright (c) 2011-2015 cPanel Inc. All rights reserved.
 # This module is free software; you can redistribute and/or modify
 # it under the same terms as Perl itself.
 
@@ -13,7 +13,7 @@
 
 package B::Bytecode;
 
-our $VERSION = '1.16';
+our $VERSION = '1.17';
 
 use 5.008;
 use B qw( class main_cv main_root main_start
@@ -42,11 +42,14 @@ BEGIN {
     B->import(qw(walkoptree));
   }
   if ($] > 5.017) {
-    B->import('SVf_IsCOW') ;
+    B->import('SVf_IsCOW');
   } else {
-    eval q[
-      sub SVf_IsCOW() {}; # unused
-     ];
+    eval q[sub SVf_IsCOW() {};]; # unused
+  }
+  if ($] > 5.021006) {
+    B->import('SVf_PROTECT');
+  } else {
+    eval q[sub SVf_PROTECT() {};]; # unused
   }
   if ( $] >= 5.017005 ) {
     @B::PAD::ISA = ('B::AV');
@@ -64,6 +67,7 @@ my $PERL512 = ( $] >= 5.011 );
 #my $PERL514 = ( $] >= 5.013002 );
 my $PERL518 = ( $] >= 5.017006 );
 my $PERL520 = ( $] >= 5.019002 );
+my $PERL522 = ( $] >= 5.021005 );
 my $DEBUGGING = ($Config{ccflags} =~ m/-DDEBUGGING/);
 our ($quiet, $includeall, $savebegins, $T_inhinc);
 my ( $varix, $opix, %debug, %walked, %files, @cloop );
@@ -74,6 +78,7 @@ my %spectab = $PERL56 ? () : ( 0, 0 ); # we need the special Nullsv on 5.6 (?)
 my $tix     = $PERL56 ? 0 : 1;
 my %ops     = ( 0, 0 );
 my @packages;    # list of packages to compile. 5.6 only
+our $curcv;
 
 # sub asm ($;$$) { }
 sub nice ($) { }
@@ -99,6 +104,10 @@ if ($PERL56) {
     require XSLoader;
     XSLoader::load('B::C'); # for op->slabbed... workarounds
   }
+  if ( $] > 5.021) { # for op_aux
+    require XSLoader;
+    XSLoader::load('B::C');
+  }
 }
 
 for ( my $i = 0 ; $i < @optype ; $i++ ) {
@@ -114,7 +123,7 @@ BEGIN {
   die $@ if $@;
 }
 
-sub as_hex {$quiet ? undef : sprintf("0x%x",shift)}
+sub as_hex($) {$quiet ? undef : sprintf("0x%x",shift)}
 
 # Fixes bug #307: use foreach, not each
 # each is not safe to use (at all). walksymtable is called recursively which might add
@@ -143,11 +152,11 @@ sub walksymtable {
 #################################################
 
 # This is for -S commented assembler output
-sub op_flags {
+sub op_flags($) {
   return '' if $quiet;
   # B::Concise::op_flags($_[0]); # too terse
   # common flags (see BASOP.op_flags in op.h)
-  my ($x) = @_;
+  my $x = shift;
   my (@v);
   push @v, "WANT_VOID"   if ( $x & 3 ) == 1;
   push @v, "WANT_SCALAR" if ( $x & 3 ) == 2;
@@ -162,11 +171,12 @@ sub op_flags {
 }
 
 # This is also for -S commented assembler output
-sub sv_flags {
+sub sv_flags($;$) {
   return '' if $quiet or $B::Concise::VERSION < 0.74;    # or ($] == 5.010);
   return '' unless $debug{Comment};
   return 'B::SPECIAL' if $_[0]->isa('B::SPECIAL');
   return 'B::PADLIST' if $_[0]->isa('B::PADLIST');
+  return 'B::PADNAMELIST' if $_[0]->isa('B::PADNAMELIST');
   return 'B::NULL'    if $_[0]->isa('B::NULL');
   my ($sv) = @_;
   my %h;
@@ -181,12 +191,12 @@ sub sv_flags {
   B::Concise::concise_sv( $_[0], \%h, 0 );
 }
 
-sub pvstring {
+sub pvstring($) {
   my $pv = shift;
   defined($pv) ? cstring( $pv . "\0" ) : "\"\"";
 }
 
-sub pvix {
+sub pvix($) {
   my $str = pvstring shift;
   my $ix  = $strtab{$str};
   defined($ix) ? $ix : do {
@@ -198,7 +208,7 @@ sub pvix {
   }
 }
 
-sub B::OP::ix {
+sub B::OP::ix($) {
   my $op = shift;
   my $ix = $optab{$$op};
   defined($ix) ? $ix : do {
@@ -252,7 +262,7 @@ sub B::OP::ix {
   }
 }
 
-sub B::SPECIAL::ix {
+sub B::SPECIAL::ix($) {
   my $spec = shift;
   my $ix   = $spectab{$$spec};
   defined($ix) ? $ix : do {
@@ -264,7 +274,7 @@ sub B::SPECIAL::ix {
   }
 }
 
-sub B::SV::ix {
+sub B::SV::ix($) {
   my $sv = shift;
   my $ix = $svtab{$$sv};
   defined($ix) ? $ix : do {
@@ -287,19 +297,76 @@ sub B::SV::ix {
   }
 }
 
-sub B::PADLIST::ix {
+#sub B::PAD::ix($) {
+#  my $sv = shift;
+#  #if ($PERL522) {
+#  #  my $ix = $svtab{$$sv};
+#  #  defined($ix) ? $ix : do {
+#  #    nice '[' . class($sv) . " $tix]";
+#  #    B::Assembler::maxsvix($tix) if $debug{A};
+#  #    asm "newpadx", 0,
+#  #      $debug{Comment} ? sprintf("pad_new(flags=0x%x)", 0) : '';
+#  #    $svtab{$$sv} = $varix = $ix = $tix++;
+#  #    $sv->bsave($ix);
+#  #    $ix;
+#  #  }
+#  #} else {
+#  if ($$sv) {
+#    bless $sv, 'B::AV';
+#    return $sv->B::SV::ix;
+#  } else {
+#    0
+#  }
+#}
+
+# since 5.18
+sub B::PADLIST::ix($) {
   my $padl = shift;
   my $ix = $svtab{$$padl};
   defined($ix) ? $ix : do {
     nice '[' . class($padl) . " $tix]";
     B::Assembler::maxsvix($tix) if $debug{A};
-    asm "newpadlx", 1;
+    asm "newpadlx", 0,
+     $debug{Comment} ? sprintf("pad_new(flags=0x%x)", 0) : '';
     $svtab{$$padl} = $varix = $ix = $tix++;
     $padl->bsave($ix);
     $ix;
   }
 }
 
+sub B::PADNAME::ix {
+  my $pn = shift;
+  my $ix = $svtab{$$pn};
+  defined($ix) ? $ix : do {
+    nice '[' . class($pn) . " $tix]";
+    B::Assembler::maxsvix($tix) if $debug{A};
+    my $pv = $pn->PVX;
+    asm "newpadnx", $pv ? cstring $pv : "";
+    $svtab{$$pn} = $varix = $ix = $tix++;
+    $pn->bsave($ix);
+    $ix;
+  }
+}
+
+sub B::PADNAMELIST::ix {
+  my $padnl = shift;
+  if (!$PERL522) {
+    return B::SV::ix(bless $padnl, 'B::AV');
+  } else {
+    my $ix = $svtab{$$padnl};
+    defined($ix) ? $ix : do {
+      nice '[' . class($padnl) . " $tix]";
+      B::Assembler::maxsvix($tix) if $debug{A};
+      my $max = $padnl->MAX;
+      asm "newpadnlx", $max,
+        $debug{Comment} ? sprintf("size=%d, %s", $max+1, sv_flags($padnl)) : '';
+      $svtab{$$padnl} = $varix = $ix = $tix++;
+      $padnl->bsave($ix);
+      $ix;
+    }
+  }
+}
+
 sub B::GV::ix {
   my ( $gv, $desired ) = @_;
   my $ix = $svtab{$$gv};
@@ -363,7 +430,7 @@ sub B::GV::ix {
       asm "gp_cvgen", $gv->CVGEN if $gv->CVGEN;
       asm "gp_form",  $formix if $formix;
       asm "gp_file",  pvix $gv->FILE;
-      asm "gp_line",  $gv->LINE;
+      asm "gp_line",  $gv->LINE if $gv->LINE;
       asm "formfeed", $svix if $name eq "main::\cL";
     }
     else {
@@ -396,12 +463,14 @@ sub B::HV::ix {
   defined($ix) ? $ix : do {
     my ( $ix, $i, @array );
     my $name = $hv->NAME;
+    my $flags = $hv->FLAGS & ~SVf_READONLY;
+    $flags &= ~SVf_PROTECT if $PERL522;
     if ($name) {
       nice "[STASH $tix]";
       B::Assembler::maxsvix($tix) if $debug{A};
       asm "gv_stashpvx", cstring $name;
       asm "ldsv", $tix if $PERL56;
-      asm "sv_flags", $hv->FLAGS, as_hex($hv->FLAGS);
+      asm "sv_flags", $flags, as_hex($flags);
       $svtab{$$hv} = $varix = $ix = $tix++;
       asm "xhv_name", pvix $name;
 
@@ -412,10 +481,11 @@ sub B::HV::ix {
     else {
       nice "[HV $tix]";
       B::Assembler::maxsvix($tix) if $debug{A};
-      asm "newsvx", $hv->FLAGS, $debug{Comment} ? sv_flags($hv) : '';
+      asm "newsvx", $flags, $debug{Comment} ? sv_flags($hv) : '';
       asm "stsv", $tix if $PERL56;
       $svtab{$$hv} = $varix = $ix = $tix++;
-      my $stashix = $hv->SvSTASH->ix;
+      my $stash = $hv->SvSTASH;
+      my $stashix = $stash ? $hv->SvSTASH->ix : 0;
       for ( @array = $hv->ARRAY ) {
         next if $i = not $i;
         $_ = $_->ix;
@@ -426,10 +496,11 @@ sub B::HV::ix {
       if ( VERSION < 5.009 ) {
         asm "xnv", $hv->NVX;
       }
-      asm "xmg_stash", $stashix;
+      asm "xmg_stash", $stashix if $stashix;
       asm( "xhv_riter", $hv->RITER ) if VERSION < 5.009;
     }
     asm "sv_refcnt", $hv->REFCNT if $hv->REFCNT != 1;
+    asm "sv_flags", $hv->FLAGS, as_hex($hv->FLAGS) if $hv->FLAGS & SVf_READONLY;
     $ix;
   }
 }
@@ -458,16 +529,18 @@ sub B::NULL::bsave {
 sub B::SV::bsave;
 *B::SV::bsave = *B::NULL::bsave;
 
-sub B::RV::bsave {
+sub B::RV::bsave($$) {
   my ( $sv, $ix ) = @_;
   my $rvix = $sv->RV->ix;
   $sv->B::NULL::bsave($ix);
   # RV with DEBUGGING already requires sv_flags before SvRV_set
-  asm "sv_flags", $sv->FLAGS, as_hex($sv->FLAGS);
+  my $flags = $sv->FLAGS;
+  $flags &= ~0x8000 if $flags & $SVt_PVGV and $PERL522; # no SVpgv_GP
+  asm "sv_flags", $flags, as_hex($flags);
   asm "xrv", $rvix;
 }
 
-sub B::PV::bsave {
+sub B::PV::bsave($$) {
   my ( $sv, $ix ) = @_;
   $sv->B::NULL::bsave($ix);
   return unless $sv;
@@ -496,7 +569,7 @@ sub B::PV::bsave {
   }
 }
 
-sub B::IV::bsave {
+sub B::IV::bsave($$) {
   my ( $sv, $ix ) = @_;
   return $sv->B::RV::bsave($ix)
     if $PERL512 and $sv->FLAGS & B::SVf_ROK;
@@ -508,13 +581,13 @@ sub B::IV::bsave {
   }
 }
 
-sub B::NV::bsave {
+sub B::NV::bsave($$) {
   my ( $sv, $ix ) = @_;
   $sv->B::NULL::bsave($ix);
   asm "xnv", sprintf "%.40g", $sv->NVX;
 }
 
-sub B::PVIV::bsave {
+sub B::PVIV::bsave($$) {
   my ( $sv, $ix ) = @_;
   if ($PERL56) {
     $sv->B::PV::bsave($ix);
@@ -545,7 +618,7 @@ sub B::PVIV::bsave {
   }
 }
 
-sub B::PVNV::bsave {
+sub B::PVNV::bsave($$) {
   my ( $sv, $ix ) = @_;
   $sv->B::PVIV::bsave($ix);
   if ($PERL510) {
@@ -558,7 +631,7 @@ sub B::PVNV::bsave {
     return if $sv->isa('B::IO');
 
     # cop_seq range instead of a double. (IV, NV)
-    unless ($sv->FLAGS & (SVf_NOK|SVp_NOK)) {
+    unless ($PERL522 or $sv->FLAGS & (SVf_NOK|SVp_NOK)) {
       asm "cop_seq_low", $sv->COP_SEQ_RANGE_LOW;
       asm "cop_seq_high", $sv->COP_SEQ_RANGE_HIGH;
       return;
@@ -567,7 +640,7 @@ sub B::PVNV::bsave {
   asm "xnv", sprintf "%.40g", $sv->NVX;
 }
 
-sub B::PVMG::domagic {
+sub B::PVMG::domagic($$) {
   my ( $sv, $ix ) = @_;
   nice1 '-MAGICAL-'; # no empty line before
   my @mglist = $sv->MAGIC;
@@ -597,16 +670,16 @@ sub B::PVMG::domagic {
   }
 }
 
-sub B::PVMG::bsave {
+sub B::PVMG::bsave($$) {
   my ( $sv, $ix ) = @_;
   my $stashix = $sv->SvSTASH->ix;
   $sv->B::PVNV::bsave($ix);
-  asm "xmg_stash", $stashix;
+  asm "xmg_stash", $stashix if $stashix;
   # XXX added SV->MAGICAL to 5.6 for compat
   $sv->domagic($ix) if $PERL56 ? MAGICAL56($sv) : $sv->MAGICAL;
 }
 
-sub B::PVLV::bsave {
+sub B::PVLV::bsave($$) {
   my ( $sv, $ix ) = @_;
   my $targix = $sv->TARG->ix;
   $sv->B::PVMG::bsave($ix);
@@ -616,7 +689,7 @@ sub B::PVLV::bsave {
   asm "xlv_type",    $sv->TYPE;
 }
 
-sub B::BM::bsave {
+sub B::BM::bsave($$) {
   my ( $sv, $ix ) = @_;
   $sv->B::PVMG::bsave($ix);
   asm "xpv_cur",      $sv->CUR if $] > 5.008;
@@ -625,7 +698,7 @@ sub B::BM::bsave {
   asm "xbm_rare",     $sv->RARE;
 }
 
-sub B::IO::bsave {
+sub B::IO::bsave($$) {
   my ( $io, $ix ) = @_;
   my $topix    = $io->TOP_GV->ix;
   my $fmtix    = $io->FMT_GV->ix;
@@ -670,8 +743,9 @@ sub B::IO::bsave {
   }
 }
 
-sub B::CV::bsave {
+sub B::CV::bsave($$) {
   my ( $cv, $ix ) = @_;
+  $B::Bytecode::curcv = $cv;
   my $stashix   = $cv->STASH->ix;
   my $gvix      = ($cv->GV and ref($cv->GV) ne 'B::SPECIAL') ? $cv->GV->ix : 0;
   my $padlistix = $cv->PADLIST->ix;
@@ -683,14 +757,14 @@ sub B::CV::bsave {
   my $xsubanyix  = ($cv->CONST and !$PERL56) ? $cv->XSUBANY->ix : 0;
 
   $cv->B::PVMG::bsave($ix);
-  asm "xcv_stash",       $stashix;
+  asm "xcv_stash",       $stashix if $stashix;
   asm "xcv_start",       $startix if $startix; # e.g. main_cv 5.18
-  asm "xcv_root",        $rootix;
-  asm "xcv_xsubany",     $xsubanyix unless $PERL56;
+  asm "xcv_root",        $rootix if $rootix;
+  asm "xcv_xsubany",     $xsubanyix if !$PERL56 and $xsubanyix;
   asm "xcv_padlist",     $padlistix;
-  asm "xcv_outside",     $outsideix;
-  asm "xcv_outside_seq", $cv->OUTSIDE_SEQ unless $PERL56;
-  asm "xcv_depth",       $cv->DEPTH;
+  asm "xcv_outside",     $outsideix if $outsideix;
+  asm "xcv_outside_seq", $cv->OUTSIDE_SEQ if !$PERL56 and $cv->OUTSIDE_SEQ;
+  asm "xcv_depth",       $cv->DEPTH if $cv->DEPTH;
   # add the RC flag if there's no backref magic. eg END (48)
   my $cvflags = $cv->CvFLAGS;
   $cvflags |= 0x400 if $] >= 5.013 and !$cv->MAGIC;
@@ -705,21 +779,24 @@ sub B::CV::bsave {
   asm "xcv_file",        pvix $cv->FILE if $cv->FILE;    # XXX AD
 }
 
-sub B::FM::bsave {
+sub B::FM::bsave($$) {
   my ( $form, $ix ) = @_;
 
   $form->B::CV::bsave($ix);
   asm "xfm_lines", $form->LINES;
 }
 
-sub B::PAD::bsave {
+# an AV or padl_sym
+sub B::PAD::bsave($$) {
   my ( $av, $ix ) = @_;
   my @array = $av->ARRAY;
   $_ = $_->ix for @array; # save the elements
   $av->B::NULL::bsave($ix);
-  # av_extend always allocs 3
-  asm "av_extend", scalar @array if @array;
-  asm "av_pushx", $_ for @array;
+  my $fill = scalar @array;
+  asm "av_extend", $fill if @array;
+  if ($fill > 1 or $array[0]) {
+    asm "av_pushx", $_ for @array;
+  }
 }
 
 sub B::AV::bsave {
@@ -760,21 +837,55 @@ sub B::AV::bsave {
     # asm "xav_alloc", $av->AvALLOC if $] > 5.013002; # XXX new but not needed
   }
   asm "sv_refcnt", $av->REFCNT if $av->REFCNT != 1;
-  asm "xmg_stash", $stashix;
+  asm "xmg_stash", $stashix if $stashix;
 }
 
+# since 5.18
 sub B::PADLIST::bsave {
   my ( $padl, $ix ) = @_;
   my @array = $padl->ARRAY;
-  bless $array[0], 'B::PAD' if ref $array[0] eq 'B::AV';
+  my $max = scalar @array;
+  bless $array[0], 'B::PADNAMELIST' if ref $array[0] eq 'B::AV';
   bless $array[1], 'B::PAD' if ref $array[1] eq 'B::AV';
-  my $ix0 = $array[0]->ix; # comppad_name
-  my $ix1 = $array[1]->ix; # comppad syms
-
+  my $pnl = $array[0]->ix; # padnamelist
+  my $pad = $array[1]->ix; # pad syms
   nice "-PADLIST-",
     asm "ldsv", $varix = $ix unless $ix == $varix;
-  asm "padl_name", $ix0 if ref $array[0] eq 'B::PAD';
-  asm "padl_sym",  $ix1 if ref $array[1] eq 'B::PAD';
+  asm "padl_name", $pnl;
+  asm "padl_sym",  $pad;
+  if ($PERL522) {
+    asm "padl_id",    $padl->id if $padl->id;
+    # 5.18-20 has no PADLIST->outid API, uses xcv_outside instead
+    asm "padl_outid", $padl->outid if $padl->outid;
+  }
+}
+
+# since 5.22
+sub B::PADNAME::bsave {
+  my ( $pn, $ix ) = @_;
+  my $stashix = $pn->OURSTASH->ix;
+  my $typeix = $pn->TYPE->ix;
+  nice "-PADNAME-",
+    asm "ldsv", $varix = $ix unless $ix == $varix;
+  asm "padn_pv", cstring $pn->PV if $pn->LEN;
+  my $flags = $pn->FLAGS;
+  asm "padn_stash", $stashix if $stashix;
+  asm "padn_type", $typeix if $typeix;
+  asm "padn_flags", $flags & 0xff if $flags & 0xff; # turn of SVf_FAKE, U8 only
+  asm "padn_seq_low", $pn->COP_SEQ_RANGE_LOW;
+  asm "padn_seq_high", $pn->COP_SEQ_RANGE_HIGH;
+  asm "padn_refcnt", $pn->REFCNT if $pn->REFCNT != 1;
+  #asm "padn_len", $pn->LEN if $pn->LEN;
+}
+
+# since 5.22
+sub B::PADNAMELIST::bsave {
+  my ( $padnl, $ix ) = @_;
+  my @array = $padnl->ARRAY;
+  $_ = $_->ix for @array;
+  nice "-PADNAMELIST-",
+    asm "ldsv", $varix = $ix unless $ix == $varix;
+  asm "padnl_push", $_ for @array;
 }
 
 sub B::GV::desired {
@@ -798,15 +909,15 @@ sub B::HV::bwalk {
   #while ( my ( $k, $v ) = each %stash )
   foreach my $k (keys %stash) {
     my $v = $stash{$k};
-    if ( !$PERL56 and $v->SvTYPE == $SVt_PVGV ) {
-      my $hash = $v->HV;
-      if ( $$hash && $hash->NAME ) {
+    if ( !$PERL56 and $v->SvTYPE == $SVt_PVGV ) { # XXX ref $v eq 'B::GV'
+      my $hash = $v->HV if $v->can("HV");
+      if ( $hash and $$hash && $hash->NAME ) {
         $hash->bwalk;
       }
       # B since 5.13.6 (744aaba0598) pollutes our namespace. Keep it clean
       # XXX This fails if our source really needs any B constant
       unless ($] > 5.013005 and $hv->NAME eq 'B') {
-	$v->ix(1) if desired $v;
+	$v->ix(1) if $v->can("desired") and desired $v;
       }
     }
     else {
@@ -846,9 +957,12 @@ sub B::OP::bsave_thin {
     if ($] >= 5.019002 and $op->can('folded')) {
       asm "op_folded", $op->folded if $op->folded;
     }
-    if ($] >= 5.021002 and $op->can('lastsib')) {
+    if ($] >= 5.021002 and $[ < 5.021011 and $op->can('lastsib')) {
       asm "op_lastsib", $op->lastsib if $op->lastsib;
     }
+    elsif ($] >= 5.021011 and $op->can('moresib')) {
+      asm "op_moresib", $op->moresib if $op->moresib;
+    }
   }
 }
 
@@ -880,13 +994,44 @@ sub B::UNOP::bsave {
   asm "op_first", $firstix;
 }
 
-sub B::BINOP::bsave {
+sub B::UNOP_AUX::bsave {
+  my ( $op, $ix ) = @_;
+  my $name    = $op->name;
+  my $flags   = $op->flags;
+  my $first   = $op->first;
+  my $firstix = $first->ix;
+  my $aux     = $op->aux;
+  my @aux_list = $op->aux_list($B::Bytecode::curcv);
+  for my $item (@aux_list) {
+    $item->ix if ref $item;
+  }
+  $op->B::OP::bsave($ix);
+  asm "op_first", $firstix;
+  asm "unop_aux", cstring $op->aux;
+}
+
+sub B::METHOP::bsave($$) {
+  my ( $op, $ix ) = @_;
+  my $name    = $op->name;
+  my $firstix = $name eq 'method' ? $op->first->ix : $op->meth_sv->ix;
+  my $rclass  = $op->rclass->ix;
+  $op->B::OP::bsave($ix);
+  if ($op->name eq 'method') {
+    asm "op_first", $firstix;
+  } else {
+    asm "methop_methsv", $firstix;
+  }
+  asm "methop_rclass", $rclass if $rclass or ITHREADS; # padoffset 0 valid threaded
+}
+
+sub B::BINOP::bsave($$) {
   my ( $op, $ix ) = @_;
   if ( $op->name eq 'aassign' && $op->private & B::OPpASSIGN_HASH() ) {
     my $last   = $op->last;
     my $lastix = do {
       local *B::OP::bsave   = *B::OP::bsave_fat;
       local *B::UNOP::bsave = *B::UNOP::bsave_fat;
+      #local *B::BINOP::bsave = *B::BINOP::bsave_fat;
       $last->ix;
     };
     asm "ldop", $lastix unless $lastix == $opix;
@@ -905,7 +1050,7 @@ sub B::BINOP::bsave {
 
 # deal with sort / formline
 
-sub B::LISTOP::bsave {
+sub B::LISTOP::bsave($$) {
   my ( $op, $ix ) = @_;
   bwarn( B::peekop($op), ", ix: $ix" ) if $debug{o};
   my $name = $op->name;
@@ -913,7 +1058,7 @@ sub B::LISTOP::bsave {
   if ( $name eq 'sort' && ( $op->flags & blocksort ) == blocksort ) {
     # Note: 5.21.2 PERL_OP_PARENT support work in progress
     my $first    = $op->first;
-    my $pushmark = $first->sibling;
+    my $pushmark = $first->sibling; # XXX may be B::NULL
     my $rvgv     = $pushmark->first;
     my $leave    = $rvgv->first;
 
@@ -933,7 +1078,7 @@ sub B::LISTOP::bsave {
     my $firstix = $first->ix;
     asm "ldop", $firstix unless $firstix == $opix;
     #asm "comment", "first" unless $quiet;
-    asm "op_sibling", $pushmarkix; # if !$first->can('lastsib') or !$first->lastsib;
+    asm "op_sibling", $pushmarkix if $first->has_sibling;
 
     $op->B::OP::bsave($ix);
     asm "op_first", $firstix;
@@ -952,17 +1097,30 @@ sub B::LISTOP::bsave {
 
 # fat versions
 
-sub B::OP::bsave_fat {
+# or parent since 5.22
+sub B::OP::has_sibling($) {
+  my $op = shift;
+  return $op->moresib if $op->can('moresib'); #5.22
+  return $op->lastsib if $op->can('lastsib'); #5.21
+  return 1;
+}
+
+sub B::OP::bsave_fat($$) {
   my ( $op, $ix ) = @_;
 
-  my $sibling = $op->sibling;
-  #if (!$op->can('lastsib') or !$op->lastsib) { # PERL_OP_PARENT
+  if ($op->has_sibling) {
+    my $sibling = $op->sibling; # might be B::NULL with 5.22 and PERL_OP_PARENT
     my $siblix = $sibling->ix;
     $op->B::OP::bsave_thin($ix);
     asm "op_sibling", $siblix;
-  #} else {
-  #  $op->B::OP::bsave_thin($ix);
-  #}
+  } elsif ($] > 5.021011 and ref($op->parent) ne 'B::NULL') {
+    my $parent = $op->parent;
+    my $pix = $parent->ix;
+    $op->B::OP::bsave_thin($ix);
+    asm "op_sibling", $pix; # but renamed to op_sibparent
+  } else {
+    $op->B::OP::bsave_thin($ix);
+  }
   # asm "op_seq", -1;			XXX don't allocate OPs piece by piece
 }
 
@@ -1277,7 +1435,9 @@ sub compile {
     *B::OP::bsave     = *B::OP::bsave_fat;
     *B::UNOP::bsave   = *B::UNOP::bsave_fat;
     *B::BINOP::bsave  = *B::BINOP::bsave_fat;
-    *B::LISTOP::bsave = *B::LISTOP::bsave_fat;
+    #*B::LISTOP::bsave = *B::LISTOP::bsave_fat;
+    #*B::LOGOP::bsave  = *B::LOGOP::bsave_fat;
+    #*B::PMOP::bsave   = *B::PMOP::bsave_fat;
   }
   sub bwarn { print STDERR "Bytecode.pm: @_\n" unless $quiet; }
 
@@ -1313,7 +1473,7 @@ use ByteLoader '$ByteLoader::VERSION';
       # Maybe: Fix the plc reader, if 'perl -MByteLoader <.plc>' is called
     }
     elsif (/^-k/) {
-      keep_syn unless $PERL510;
+      keep_syn() if !$PERL510 or $PERL522;
     }
     elsif (/^-m/) {
       $module = 1;
@@ -1407,6 +1567,7 @@ use ByteLoader '$ByteLoader::VERSION';
     save_init_end;
 
     unless ($module) {
+      $B::Bytecode::curcv = main_cv;
       nice '<main_start>';
       asm "main_start", $PERL56 ? main_start->ix : main_start->opwalk;
       #asm "main_start", main_start->opwalk;
