@@ -3,6 +3,7 @@
 
 use Test::More;
 use File::Spec;
+use B::C::Flags;
 
 sub curr_test {
     $test = shift if @_;
@@ -602,6 +603,7 @@ sub run_cc_test {
     ($fnbackend,$opt) = $fnbackend =~ /^(cc?)(,-o.)?/;
     $opt =~ s/,-/_/ if $opt;
     $opt = '' unless $opt;
+    #if ($] > 5.021006 and $fnbackend eq 'cc') { print "ok $cnt # skip CC for 5.22 WIP\n"; return 1; }
     use Config;
     require B::C::Flags;
     # note that the smokers run the c.t and c_o3.t tests in parallel, with possible
@@ -659,7 +661,9 @@ sub run_cc_test {
         my $linkargs = ExtUtils::Embed::ldopts('-std');
         # At least cygwin gcc-4.3 crashes with 2x -fstack-protector
         $linkargs =~ s/-fstack-protector\b//
-          if $command =~ /-fstack-protector\b/ and $linkargs =~ /-fstack-protector\b/;
+          if $linkargs !~ /-fstack-protector-strong\b/
+          and $command =~ /-fstack-protector\b/
+          and $linkargs =~ /-fstack-protector\b/;
 	if ( -e "$coredir/$Config{libperl}" and $Config{libperl} !~ /\.$so$/) {
 	    $command .= $linkargs;
 	} elsif ( $useshrplib and (-e "$libdir/$Config{libperl}" or -e "/usr/lib/$Config{libperl}")) {
@@ -847,6 +851,10 @@ sub plctestok {
 sub plctest {
     my ($num, $expected, $base, $script, $todo) =  @_;
 
+    if ($] > 5.021006 and !$B::C::Flags::have_byteloader) {
+        ok(1, "SKIP perl5.22 broke ByteLoader");
+        return 1;
+    }
     my $name = $base."_$num";
     unlink($name, "$name.plc", "$name.pl", "$name.exe");
     open F, ">", "$base.pl";
@@ -864,8 +872,8 @@ sub plctest {
     system $cmd;
     # $out =~ s/^$base.pl syntax OK\n//m;
     unless (-e "$name.plc") {
-        print "not ok $num #B::Bytecode failed\n";
-        exit;
+        ok(0, '#B::Bytecode failed');
+        return 1;
     }
     $cmd = "$runperl $Mblib -MByteLoader $name.plc";
     diag($cmd) if $ENV{TEST_VERBOSE} and $ENV{TEST_VERBOSE} > 1;
@@ -896,6 +904,7 @@ sub ctest {
     my ($num, $expected, $backend, $base, $script, $todo) =  @_;
     my $name = $base."_$num";
     my $b = $backend; # protect against parallel test name clashes
+    #if ($] > 5.021006 and $backend =~ /^CC/i) { ok(1, "skip CC for 5.22 WIP"); return 1; } # temp 5.22
     $b =~ s/-(D.*|f.*|v),//g;
     $b =~ s/-/_/g;
     $b =~ s/[, ]//g;
@@ -918,8 +927,8 @@ sub ctest {
       if $ENV{TEST_VERBOSE} and $ENV{TEST_VERBOSE} > 1;
     system "$runperl ".Mblib." -MO=$b,-o$name.c $post $name.pl";
     unless (-e "$name.c") {
-        print "not ok $num #B::$backend failed\n";
-        exit;
+        ok (undef, "#B::$backend failed");
+        return 1;
     }
     diag("$runperl ".Mblib." blib/script/cc_harness -q -o $name $name.c")
       if $ENV{TEST_VERBOSE} and $ENV{TEST_VERBOSE} > 1;
@@ -998,8 +1007,8 @@ sub ccompileok {
     my $Mblib = Mblib;
     system "$runperl $Mblib -MO=$b,-o$name.c $name.pl";
     unless (-e "$name.c") {
-        print "not ok 1 #B::$backend failed\n";
-        exit;
+        ok (undef, "#B::$backend failed");
+        return 1;
     }
     system "$runperl $Mblib blib/script/cc_harness -q -o $name $name.c";
     my $ok = -e $name or -e "$name.exe";
@@ -1033,8 +1042,15 @@ sub todo_tests_default {
 	push @todo, (41)  if $] < 5.007; #regressions
         push @todo, (12)  if $what eq 'c_o3' and !$ITHREADS and $] >= 5.008009 and $] < 5.010;
 
-        push @todo, (48)  if $what eq 'c_o4' and $ITHREADS;
+        push @todo, (48)  if $what eq 'c_o4' and $] < 5.021 and $ITHREADS;
         push @todo, (8,18,19,25,26,28)  if $what eq 'c_o4' and !$ITHREADS;
+        push @todo, (13,18,29,34) if $] >= 5.021006 and $ITHREADS;
+        push @todo, (10,15,27,41,42,43,44,45,49,50)
+          if $] >= 5.021006 and $what eq 'c_o4';
+        push @todo, (13,18,29,34)
+          if $] >= 5.021006 and $what eq 'c_o4' and $ITHREADS;
+        push @todo, (12,14,38)
+          if $] >= 5.021006 and $what eq 'c_o4' and !$ITHREADS;
     } elsif ($what =~ /^cc/) {
 	push @todo, (21,30,105,106);
 	push @todo, (22,41,45,103) if $] < 5.007; #regressions
@@ -1050,9 +1066,12 @@ sub todo_tests_default {
 	push @todo, (12)    if $^O eq 'MSWin32' and $Config{cc} =~ /^cl/i;
 	push @todo, (26)    if $what =~ /^cc_o[12]/;
         push @todo, (27)    if $] > 5.008008 and $] < 5.009;
-	push @todo, (27)    if $] > 5.008008 and $] < 5.009 and $what eq 'cc_o2';
+	#push @todo, (27)    if $] > 5.008008 and $] < 5.009 and $what eq 'cc_o2';
         push @todo, (103)   if ($] >= 5.012 and $] < 5.014 and !$ITHREADS);
         push @todo, (12,19,25) if $] >= 5.019;
+        push @todo, (25)    if $] >= 5.021006;
+	push @todo, (29)    if $] >= 5.021006 and $what eq 'cc_o1';
+	push @todo, (24,29) if $] >= 5.021006 and $what eq 'cc_o2';
     }
     push @todo, (48)   if $] > 5.007 and $] < 5.009 and $^O =~ /MSWin32|cygwin/i;
     return @todo;
