@@ -15,16 +15,22 @@ function help {
   echo " -o                 orig. no -Mblib, use installed modules (5.6, 5.8)"
   echo " -a                 all. undo -Du. Unsilence scanning unused sub"
   echo " -A                 -DALLOW_PERL_OPTIONS"
+  echo " -L                 make CORE test symlinks and exit"
+  echo " -X<num>            view the test"
   echo " -q                 quiet"
   echo " -h                 help"
   echo "Without arguments try all $ntests tests. Without Option -Ox try -O0 to -O3 optimizations."
 }
 
-# use the actual perl from the Makefile (perl5.8.8, 
+# use the actual perl from the Makefile (perl5.8.8,
 # perl5.10.0d-nt, perl5.11.0, ...)
 PERL=`grep "^PERL =" Makefile|cut -c8-`
 PERL=${PERL:-perl}
+PERL=`echo $PERL|sed -e's,^",,; s,"$,,'`
+v510=`$PERL -e'print (($] < 5.010)?0:1)'`
 v518=`$PERL -e'print (($] < 5.018)?0:1)'`
+PERLV=v5.`$PERL -e'print substr($],3,2)'`
+XTESTC="t/CORE/$PERLV/C-COMPILED/xtestc"
 
 function init {
 BASE=`basename $0`
@@ -34,9 +40,9 @@ Mblib=${Mblib:--Iblib/arch -Iblib/lib} # B::C is now fully 5.6+5.8 backwards com
 v513="`$PERL -e'print (($] < 5.013005) ? q() : q(-fno-fold,-fno-warnings,))'`"
 # OCMD=${OCMD}${v513}
 if [ -z "$Mblib" ]; then
-    VERS="${VERS}_global"; 
+    VERS="${VERS}_global";
     OCMD="$PERL $Mblib -MO=C,${v513}-Dcsp,"
-    if [ $BASE = "testcc.sh" ]; then # DrOsplt 
+    if [ $BASE = "testcc.sh" ]; then # DrOsplt
         OCMD="$PERL $Mblib -MO=CC,${v513}-DOsplt,"
     fi
 else
@@ -110,16 +116,48 @@ function runopt {
     fi
 }
 
+function emit_test {
+  n=$1
+  CONTENT="${tests[${n}]}"
+  if [ "x$CONTENT" != "x" ]; then
+    echo -E "$CONTENT"
+    echo
+    if [ "x${result[$n]}" = "x" ]; then result[$n]='ok'; fi
+    echo -E "### RESULT:${result[$n]}"
+  fi
+}
+
+function make_t_symlink {
+  n=$1
+  CONTENT="${tests[${n}]}"
+  if [ "x$CONTENT" != "x" ]; then
+    FILE_NUM=$(printf "%04d" $n)
+    FILE="$XTESTC/${FILE_NUM}.t"
+    test -e $FILE && unlink $FILE
+    ln -s ../testc.pl $FILE
+  fi
+}
+
+function make_symlinks {
+  MAX=9999
+  test -d $XTESTC || mkdir $XTESTC
+  rm -f $XTESTC/*.t ||:
+  for b in $(seq $MAX); do
+    make_t_symlink $b
+  done
+}
+
 function ctest {
     n=$1
     str=$2
-    if [ $BASE = "testcc.sh" ]; then 
+
+    if [ $BASE = "testcc.sh" ]; then
       o="cccode$n"
     else
       o="ccode$n"
     fi
     if [ -z "$str" ]; then
-        if [ "$n" = "08" ]; then n=8; fi 
+        if [ "$n" = "08" ]; then n=8; fi
         if [ "$n" = "09" ]; then n=9; fi
 	echo "${tests[${n}]}" > ${o}.pl
         str="${tests[${n}]}"
@@ -154,7 +192,7 @@ function ctest {
 	      runopt $o 1 && \
 	        runopt $o 2 && \
 	        runopt $o 3 && \
-	        runopt $o 4
+	        true #runopt $o 4
             fi
 	    true
 	else
@@ -165,7 +203,7 @@ function ctest {
     fi
 }
 
-ntests=300
+ntests=3000
 declare -a tests[$ntests]
 declare -a result[$ntests]
 ncctests=23
@@ -193,10 +231,10 @@ tests[8]='sub AUTOLOAD { print 1 } &{"a"}()'
 result[8]='1'
 tests[9]='my $l_i = 3; $x = sub { print $l_i }; &$x'
 result[9]='3'
-tests[10]='my $i_i = 1; 
+tests[10]='my $i_i = 1;
 my $foo = sub {
   $i_i = shift if @_
-}; print $i_i; 
+}; print $i_i;
 print &$foo(3),$i_i;'
 result[10]='133'
 # index: do fbm_compile or not
@@ -249,8 +287,11 @@ result[26]="26"
 tests[27]='use Fcntl ();my $a=Fcntl::O_CREAT(); print "ok" if ( $a >= 64 && &Fcntl::O_CREAT >= 64 );'
 # require $fname
 tests[28]='my($fname,$tmp_fh);while(!open($tmp_fh,">",($fname=q{ccode28_} . rand(999999999999)))){$bail++;die "Failed to create a tmp file after 500 tries" if $bail>500;}print {$tmp_fh} q{$x="ok";1;};close($tmp_fh);sleep 1;require $fname;END{unlink($fname);};print $x;'
+# multideref with static index and sv and dynamic gv ptrs
+tests[29]='my (%b,%h); BEGIN { %b=(1..8);@a=(1,2,3,4); %h=(1=>2,3=>4) } $i=0; my $l=-1; print $h->{$b->{3}},$h->{$a[-1]},$a[$i],$a[$l],$h{3}'
+result[29]='144'
 # special old IO handling
-tests[29]='use IO;print "ok"'
+tests[291]='use IO;print "ok"'
 # run-time context of .., fails in CC
 tests[30]='@a=(4,6,1,0,0,1);sub range{(shift @a)..(shift @a)}print range();while(@a){print scalar(range())}'
 result[30]='456123E0'
@@ -314,12 +355,12 @@ tests[50]='package Top;sub top{q(ok)};package Next;our @ISA=qw(Top);package main
 # XXX TODO sigwarn $w = B::NULL without -v
 tests[51]='$SIG{__WARN__}=sub{print "ok"};warn 1;'
 # check if general signals work
-tests[511]='BEGIN{$SIG{USR1}=sub{$w++;};} kill USR1 => $$; print q(ok) if $w';
+tests[511]='BEGIN{$SIG{USR1}=sub{$w++;};} kill USR1 => $$; print q(ok) if $w'
 #-------------
 # issue27
 tests[527]='require LWP::UserAgent;print q(ok);'
 #issue 24
-tests[124]='dbmopen(%H,q(f),0644);print q(ok);'
+tests[124]='my %H;dbmopen(%H,q(f),0644);print q(ok);'
 tests[68]='package A;
 sub test {
   use Data::Dumper ();
@@ -331,10 +372,10 @@ print "ok"'
 tests[71]='
 package my;
 our @a;
-sub f { 
+sub f {
   my($alias,$name)=@_;
   unshift(@a, $alias => $name);
-  my $find = "ok"; 
+  my $find = "ok";
   my $val = $a[1];
   if ( ref($alias) eq "Regexp" && $find =~ $alias ) {
     eval $val;
@@ -362,7 +403,7 @@ tests[76]='use warnings;
   length "ok";
   print "ok"
 };'
-tests[81]='sub int::check {1}  #create int package for types
+tests[81]='%int::;  #create int package for types
 sub x(int,int) { @_ } #cvproto
 my $o = prototype \&x;
 if ($o eq "int,int") {print "o"}else{print $o};
@@ -496,7 +537,7 @@ tests[99]='package my;sub recurse{my $i=shift;recurse(++$i)unless $i>5000;print"
 if [[ $v518 -gt 0 ]]; then
   tests[130]='no warnings "experimental::lexical_subs";use feature "lexical_subs";my sub p{q(ok)}; my $a=\&p;print p;'
 fi
-tests[135]='"to" =~ /t(?{ print "ok"})o/;'
+tests[137]='"to" =~ /t(?{ print "ok"})o/;'
 tests[138]='print map { chr $_ } qw/97 98 99/;'
 result[138]='abc'
 tests[140]='my %a;print "ok" if !%a;'
@@ -633,9 +674,7 @@ tests[172]='package Foo;
 use overload q("") => sub { "Foo" };
 package main;
 my $foo = bless {}, "Foo";
-print "ok " if "$foo" eq "Foo";
-print "$foo\n";'
-result[172]='ok Foo'
+print "ok\n" if "$foo" eq "Foo";'
 tests[173]='# WontFix
 use constant BEGIN   => 42; print "ok 1\n" if BEGIN == 42;
 use constant INIT   => 42; print "ok 2\n" if INIT == 42;
@@ -738,7 +777,10 @@ tests[192]='use warnings;
  }}->($h{k});
 }'
 tests[193]='unlink q{not.a.file}; $! = 0; open($FOO, q{not.a.file}); print( $! ne 0 ? "ok" : q{error: $! should not be 0}."\n"); close $FOO;'
-tests[194]='$0 = q{ccdave}; #print "pid: $$\n";
+tests[194]='$0 = q{ccdave with long name}; #print "pid: $$\n";
+$s=`ps w | grep "$$" | grep "[c]cdave"`;
+print ($s =~ /ccdave with long name/ ? q(ok) : $s);'
+tests[1941]='$0 = q{ccdave}; #print "pid: $$\n";
 $s=`ps auxw | grep "$$" | grep "ccdave"|grep -v grep`;
 print q(ok) if $s =~ /ccdave/'
 # duplicate of 152
@@ -777,7 +819,8 @@ result[198]='23'
 # duplicate of 90
 tests[199]='"abc" =~ /(.)./; print @+; print "end\n"'
 result[199]='21end'
-tests[200]='%u=("\x{123}"=>"fo"); print "ok" if $u{"\x{123}"} eq "fo"'
+tests[200]='use Encode::Byte; use Encode::KR; print "ok\n"'
+tests[2000]='%u=("\x{123}"=>"fo"); print "ok" if $u{"\x{123}"} eq "fo"'
 tests[2001]='BEGIN{%u=("\x{123}"=>"fo");} print "ok" if $u{"\x{123}"} eq "fo";'
 tests[201]='use Storable;*Storable::CAN_FLOCK=sub{1};print qq{ok\n}'
 tests[2011]='sub can {require Config; import Config;return $Config{d_flock}}
@@ -851,15 +894,34 @@ tests[215]='eval { $@ = "t1\n"; do { die "t3\n" }; 1; }; print ":$@:\n";'
 result[215]=':t3
 :'
 tests[216]='eval { $::{q{@}}=42; }; print qq{ok\n}'
-# also at 904
-tests[220]='
-my $content = "ok\n";
-while ( $content =~ m{\w}g ) {
-    $_ .= "$-[0]$+[0]";
+# priority, fails since 5.18
+tests[219]='package OverloadTest; use overload qw("") => sub { ${$_[0]} }; package main;
+my $foo = bless \(my $bar = "ok"), "OverloadTest"; print $foo."\n";'
+tests[2190]='package Foo; use overload; sub import { overload::constant "integer" => sub { return shift }}; package main; BEGIN { $INC{"Foo.pm"} = "/lib/Foo.pm" }; use Foo; my $result = eval "5+6"; print "$result\n"'
+result[2190]='11'
+# old issue 220 see 904
+tests[220]='BEGIN { $^H{dooot} = 1 }
+sub hint_fetch {
+    my $key = shift;
+    my @results = caller(0);
+    $results[10]->{$key};
 }
-print "ok" if $_ eq "0112";'
-tests[223]='use strict; eval q({ $x = sub }); print $@'
-result[223]='Illegal declaration of anonymous subroutine at (eval 1) line 1.'
+print qq{ok\n} if hint_fetch("dooot");'
+tests[2201]='BEGIN { $^H{dчастt} = 1 }
+sub hint_fetch {
+    my $key = shift;
+    my @results = caller(0);
+    $results[10]->{$key};
+}
+print qq{ok\n} if hint_fetch("dчастt");'
+tests[2231]='use strict; eval q({ $x = sub }); print $@'
+result[2231]='Illegal declaration of anonymous subroutine at (eval 1) line 1.'
+tests[222]='my $qr = qr/(?{<<END})/;
+boom
+END
+print "ok";
+'
+tests[223]='<*> and print qq{ok\n}'
 tests[224]='use bytes; my $p = "\xB6"; my $u = "\x{100}"; my $pu = "\xB6\x{100}"; print ( $p.$u eq $pu ? "ko\n" : "ok\n" );'
 tests[225]='$_ = $dx = "\x{10f2}"; s/($dx)/$dx$1/; $ok = 1 if $_ eq "$dx$dx"; $_ = $dx = "\x{10f2}"; print qq{end\n};'
 result[225]='end'
@@ -892,6 +954,8 @@ $test
 }
 f("");
 '
+tests[2381]='sub is { $_[0] eq $_[1] and print "ok\n"}
+use constant INIT => 5; is(INIT, 5)'
 tests[239]='my $x="1";
 format STDOUT =
 ok @<<<<<<<
@@ -913,45 +977,48 @@ tests[244]='print "($_)\n" for q{-2}..undef;'
 result[244]='(-2)
 (-1)
 (0)'
-tests[245]='sub foo {
+tests[245]='%INC = (); require XSLoader; XSLoader::load("Cwd"); print qq{ok}'
+tests[2450]='sub foo {
     my ( $a, $b ) = @_;
     print "a: ".ord($a)." ; b: ".ord($b)." [ from foo ]\n";
 }
 print "a: ". ord(lc("\x{1E9E}"))." ; ";
 print "b: ". ord("\x{df}")."\n";
 foo(lc("\x{1E9E}"), "\x{df}");'
-result[245]='a: 223 ; b: 223
+result[2450]='a: 223 ; b: 223
 a: 223 ; b: 223 [ from foo ]'
+tests[246]='no warnings "experimental::const_attr"; print qq{ok} if &{sub () : const { 42 }} == 42;'
 # see t/issue235.t test 2
-tests[246]='sub foo($\@); eval q/foo "s"/; print $@'
-result[246]='Not enough arguments for main::foo at (eval 1) line 2, at EOF'
+tests[2460]='sub foo($\@); eval q/foo "s"/; print $@'
+result[2460]='Not enough arguments for main::foo at (eval 1) line 1, at EOF'
 tests[247]='# WontFix
 no warnings; $[ = 1; $big = "N\xabN\xab"; print qq{ok\n} if rindex($big, "N", 3) == 3'
 tests[248]='#WONTFIX lexical $_ in re-eval
 {my $s="toto";my $_="titi";{$s =~ /to(?{ print "-$_-$s-\n";})to/;}}'
 result[248]='-titi-toto-'
-tests[249]='#TODO version
-use version; print version::is_strict(q{01}) ? 1 : 0'
-result[249]='0'
-tests[250]='#TODO version
+tests[249]='use version; print version::is_strict(q{01}) ? 1 : q(ok)'
+tests[2501]='#TODO version
 use warnings qw/syntax/; use version; $withversion::VERSION = undef; eval q/package withversion 1.1_;/; print $@;'
-result[250]='Misplaced _ in number at (eval 1) line 1.
+result[2501]='Misplaced _ in number at (eval 1) line 1.
 Invalid version format (no underscores) at (eval 1) line 1, near "package withversion "
 syntax error at (eval 1) line 1, near "package withversion 1.1_"'
+if [[ $v518 -gt 0 ]]; then
+  tests[250]='use feature q/evalbytes/; print "ok\n" if evalbytes("1+7") == 8'
+fi
 tests[251]='sub f;print "ok" if exists &f'
 tests[2511]='#TODO 5.18
 sub f :lvalue;print "ok" if exists &f'
 tests[2512]='sub f ();print "ok" if exists &f'
 tests[2513]='sub f ($);print "ok" if exists &f'
 tests[2514]='sub f;print "ok" if exists &f'
-# duplicate of 234
-tests[252]='my $i = 0; for ("-3".."0") { ++$i } print $i'
-result[252]='4'
-tests[253]='INIT{require "t/test.pl"}plan(tests=>2);is("\x{2665}", v9829);is(v9829,"\x{2665}");'
-result[253]='1..2
+tests[252]='package bar; sub search { shift =~ m?bar? ? 1 : 0 } sub reset_zlopp { reset } package foo; sub ZZIP { shift =~ m?ZZIP? ? 1 : 0 } package main; foo::ZZIP("ZZIP"); bar::reset_zlopp(); !foo::ZZIP("ZZIP") and print "ok"'
+tests[253]='use Unicode::UCD q/prop_invmap/; my @list = prop_invmap("Uppercase_Mapping"); print "ok"'
+tests[2530]='INIT{require "t/test.pl"}plan(tests=>2);is("\x{2665}", v9829);is(v9829,"\x{2665}");'
+result[2530]='1..2
 ok 1
 ok 2'
-tests[254]='#TODO destroy upgraded lexvar
+tests[254]='Foo->UNIVERSAL::can("boogie"); print "ok" unless eval q/Foo->boogie(); 1/;'
+tests[2540]='#TODO destroy upgraded lexvar
 my $flag = 0;
 sub  X::DESTROY { $flag = 1 }
 {
@@ -981,54 +1048,97 @@ tests[263]='use JSON::XS; print encode_json []'
 result[263]='[]'
 tests[264]='no warnings; warn "$a.\n"'
 result[264]='.'
+tests[269]='use constant roref => \2; eval { for (roref) { $_ = 42 } }; print $@'
+tests[270]='*x = *STDOUT; print {*x{IO}} "ok\n";'
+tests[271]='my $FALSE = 0;
+END { delete $ENV{"Boom"} if $FALSE }
+
+my $kid = open my $fh, "-|";
+if ($kid) { # parent
+    my $read = <$fh>;
+    close($fh) or die "cannot close pipe from kid proc: $!";
+    print "ok\n";
+}
+else { # child
+    print "$$\n";
+    exit;
+}'
 tests[272]='$d{""} = qq{ok\n}; print $d{""};'
 tests[2721]='BEGIN{$d{""} = qq{ok\n};} print $d{""};'
-tests[273]='package Foo; use overload; sub import { overload::constant "integer" => sub { return shift }}; package main; BEGIN { $INC{"Foo.pm"} = "/lib/Foo.pm" }; use Foo; my $result = eval "5+6"; print "$result\n"'
-result[273]='11'
+tests[273]='package _charnames; sub foo { ($name =~ /^(\p{_Perl_Charname_Begin})/) and return; } print "ok\n";'
+tests[2731]='print "ok" if "\N{KELVIN SIGN}" eq "\N{KELVIN SIGN}"'
 tests[274]='package Foo;
-
 sub match { shift =~ m?xyz? ? 1 : 0; }
 sub match_reset { reset; }
-
 package Bar;
-
 sub match { shift =~ m?xyz? ? 1 : 0; }
 sub match_reset { reset; }
-
 package main;
 print "1..5\n";
-
 print "ok 1\n" if Bar::match("xyz");
 print "ok 2\n" unless Bar::match("xyz");
 print "ok 3\n" if Foo::match("xyz");
 print "ok 4\n" unless Foo::match("xyz");
-
 Foo::match_reset();
-print "ok 5\n" if Foo::match("xyz");'
+print "ok 5\n" if Foo::match("xyz");
+print "ok 6\n" if !Bar::match("xyz");
+'
 result[274]='1..5
 ok 1
 ok 2
 ok 3
 ok 4
 ok 5'
-tests[277]='format OUT =
+# ignored xop
+tests[2740]='use Devel::Peek; my %hash = ( a => 1 ); Dump(%hash) if $ENV{FALSE}; print "ok\n"'
+# call xop (failed with -O1)
+tests[2741]='use Devel::Peek; my %hash = ( a => 1 ); Dump(%hash); print "ok\n"'
+if [[ $v518 -gt 0 ]]; then
+  tests[276]='sub t2 : lvalue; print qq/ok\n/'
+fi
+tests[277]='sub t2 : lvalue; print "ok"'
+tests[2770]='format OUT =
 bar ~~
 .
 open(OUT, ">/dev/null"); write(OUT); close OUT; print q(ok)'
-tests[280]='package M; $| = 1; sub DESTROY {eval {print "Farewell ",ref($_[0])};} package main; bless \$A::B, q{M}; *A:: = \*B::;'
-result[280]='Farewell M'
-tests[281]='"I like pie" =~ /(I) (like) (pie)/; "@-" eq  "0 0 2 7" and print "ok\n"; print "\@- = @-\n\@+ = @+\nlen \@- = ",scalar @-'
-result[281]='ok
+tests[278]='my $ok; sub X::DESTROY { $ok = 1 } { my $x; BEGIN { $x = 42 } $x = bless {}, "X"; } print qq/ok\n/ if $ok;'
+tests[279]='*TIESCALAR = sub {}; tie my $var => "main", 42; <${var}>; print qq/ok\n/'
+tests[280]='my $z=0; my $li2="c"; my $rh={foo=>["ok"]}; print $rh->{"foo"}->[$li2+$z];'
+tests[2800]='package M; $| = 1; sub DESTROY {eval {print "Farewell ",ref($_[0])};} package main; bless \$A::B, q{M}; *A:: = \*B::;'
+result[2800]='Farewell M'
+tests[2811]='"I like pie" =~ /(I) (like) (pie)/; "@-" eq  "0 0 2 7" and print "ok\n"; print "\@- = @-\n\@+ = @+\nlen \@- = ",scalar @-'
+result[2811]='ok
 @- = 0 0 2 7
 @+ = 10 1 6 10
 len @- = 4'
+if [[ $v518 -gt 0 ]]; then
+    tests[281]='# nested formats >5.18
+open(NEST, ">Op_write.tmp");
+format NEST =
+@<<<
+{
+    my $birds = "birds";
+    local *NEST = *BIRDS{FORMAT};
+    write NEST;
+    format BIRDS =
+@<<<<<
+$birds;
+.
+    "nest"
+}
+.
+write NEST; close NEST;
+print `cat Op_write.tmp`;'
+    result[281]='birds
+nest'
+fi
 tests[282]='use vars qw($glook $smek $foof); $glook = 3; $smek = 4; $foof = "halt and cool down"; my $rv = \*smek; *glook = $rv; my $pv = ""; $pv = \*smek; *foof = $pv; print "ok\n";'
 tests[283]='#238 Undefined format "STDOUT"
 format =
 ok
 .
 write'
-tests[284]='#-O3 only
+tests[2841]='#-O3 only
 my $x="123456789";
 format OUT =
 ^<<~~
@@ -1038,29 +1148,75 @@ open OUT, ">ccode.tmp";
 write(OUT);
 close(OUT);
 print `cat "ccode.tmp"`'
-result[284]='123
+result[2841]='123
 456
 789'
+
+# issue 287 with Inf and NaN
+tests[2870]='my $i = "Inf" + 0; print $i <= 0 ? "not $i " : "", "ok\n";'
+tests[2871]='my $i = "-Inf" + 0; print $i >= 0 ? "not $i " : "", "ok\n";'
+tests[2872]='my $i = "NaN" + 0; print $i <= 0 ? "not $i " : "", "ok\n"'
+
+tests[284]='use Encode; find_encoding("euc-jp") and print qq/ok\n/'
+# mojibake
+tests[288]='use utf8; package Ｄiᚪၚd_A; sub ᕘ { "A" } package Ｄiᚪၚd_B; our @ISA = ("Ｄiᚪၚd_A"); sub ᕘ { "B => " . (shift)->SUPER::ᕘ } package Ｄiᚪၚd_C; our @ISA = ("Ｄiᚪၚd_B"); sub ᕘ { "C => " . (shift)->SUPER::ᕘ } Ｄiᚪၚd_C->ᕘ eq "C => B => A" and print "ok";'
 tests[289]='no warnings; sub z_zwap (&); print qq{ok\n} if eval q{sub z_zwap {return @_}; 1;}'
-tests[290]='sub f;print "ok" if exists &f && not defined &f;'
+tests[2901]='sub f;print "ok" if exists &f && not defined &f;'
+tests[290]='print "ok\n"if "IO::File" eq ref *STDOUT{IO}'
 tests[293]='use Coro; print q(ok)'
-tests[295]='"zzaaabbb" =~ m/(a+)(b+)/ and print "@- : @+\n"'
-result[295]='2 2 5 : 8 5 8'
-tests[299]='#TODO version
+tests[294]='#!perl -w
+BEGIN { $SIG{__WARN__} = sub { my $s = shift; do { warn $s; die $s } if $s =~ qr{Constant subroutine.*redefined}i }; }
+use File::Glob;
+File::Glob->can("XXX")->() if $ENV{ABCD};
+print qq/ok\n/'
+if [[ $v518 -gt 0 ]]; then
+  tests[295]='my @a = qw/ok/; my @to = (); s/(\w)(?{push @to, $1})/,$1,/g for @a; print "ok\n" if "@to" eq "o k";'
+fi
+tests[2950]='"zzaaabbb" =~ m/(a+)(b+)/ and print "@- : @+\n"'
+result[2950]='2 2 5 : 8 5 8'
+if [[ $v510 -gt 0 ]]; then
+  tests[298]='
+package D1; sub testmeth { "wrong" }
+package C1; our @ISA = qw/D1/; sub testmeth { "right" }
+package B1; our @ISA = qw/D1/;
+package A1; use mro "c3"; our @ISA = qw/B1 C1/; sub testmeth { shift->next::method }
+A1->testmeth() eq "right" and print "ok\n"'
+fi
+if [[ $v518 -gt 0 ]]; then
+  tests[299]='no warnings qw{experimental::lexical_topic}; my $s = "ok\n"; my $_ = "not ok\n"; my $r = $s =~ /ok(?{ print qq[$_] })/;'
+fi
+tests[2990]='#TODO version
 package Pickup; use UNIVERSAL qw( VERSION ); print qq{ok\n} if VERSION "UNIVERSAL";'
-tests[300]='use mro;print @{mro::get_linear_isa("mro")};'
-result[300]='mro'
-tests[301]='{ package A; use mro "c3";  sub foo { "A::foo" } } { package B; use base "A"; use mro "c3"; sub foo { (shift)->next::method() } } print qq{ok\n} if B->foo eq "A::foo";'
-tests[305]='use constant ASCII => eval { require Encode; Encode::find_encoding("ascii"); } || 0; print ASCII->encode("www.google.com")'
-result[305]='www.google.com'
+tests[300]='format STDERR =
+.
+my $stdout = *STDOUT{IO};
+my $stderr = *STDERR{FORMAT};
+print ref($stdout).q/ || /.ref($stderr)'
+result[300]='IO::File || FORMAT'
+tests[3000]='use mro;print @{mro::get_linear_isa("mro")};'
+result[3000]='mro'
+tests[3010]='{ package A; use mro "c3";  sub foo { "A::foo" } } { package B; use base "A"; use mro "c3"; sub foo { (shift)->next::method() } } print qq{ok\n} if B->foo eq "A::foo";'
+tests[301]='use utf8; use warnings; sub Ṩp맅싵Ş { "foo" } sub abcd { "bar" } my $w; $SIG{__WARN__} = sub { $w = $_[0] }; *{"Ṩp맅싵Ş"} = \&{"xyz"}; print "W1" if $w; $w = ""; *{"abcd"} = \&{"xyz"}; print "W2" if $w;'
+result[301]="W1W2"
+if [[ $v518 -gt 0 ]]; then
+  tests[302]='use feature "say"; eval q{say "ok"}; print $@ if($@);'
+  tests[304]='no warnings; use feature "lexical_subs"; my sub a; print qq/ok\n/'
+  tests[305]='my $gen = sub { sub () { 8 } }; my $sub = &$gen; print qq/ok\n/'
+fi
+tests[3050]='use constant ASCII => eval { require Encode; Encode::find_encoding("ascii"); } || 0; print ASCII->encode("www.google.com")'
+result[3050]='www.google.com'
 tests[3051]='INIT{ sub ASCII { eval { require Encode; Encode::find_encoding("ASCII"); } || 0; }} print ASCII->encode("www.google.com")'
 result[3051]='www.google.com'
 tests[3052]='use Net::DNS::Resolver; my $res = Net::DNS::Resolver->new; $res->send("www.google.com"), print q(ok)'
-tests[3053]='use constant JP => eval { require Encode; Encode::find_encoding("euc-jp"); } || 0; print JP->encode("www.google.com")'
-result[3053]='www.google.com'
+tests[365]='use constant JP => eval { require Encode; Encode::find_encoding("euc-jp"); } || 0; print JP->encode("www.google.com")'
+result[365]='www.google.com'
 tests[306]='package foo; sub check_dol_slash { print ($/ eq "\n" ? "ok" : "not ok") ; print  "\n"} sub begin_local { local $/;} ; package main; BEGIN { foo::begin_local() }  foo::check_dol_slash();'
 tests[308]='print (eval q{require Net::SSLeay;} ? qq{ok\n} : $@);'
-tests[309]='print $_,": ",(eval q{require }.$_.q{;} ? qq{ok\n} : $@) for qw(Net::LibIDN Net::SSLeay);'
+tests[309]='#-O0 only
+sub Regexp::DESTROY() { print qq/ok\n/ } my $rx = qr//; undef($rx)'
+tests[3090]='print $_,": ",(eval q{require }.$_.q{;} ? qq{ok\n} : $@) for qw(Net::LibIDN Net::SSLeay);'
+result[3090]='Net::LibIDN: ok
+Net::SSLeay: ok'
 tests[310]='package foo;
 sub dada { my $line = <DATA> }
 print dada;
@@ -1079,11 +1235,41 @@ package Diamond_C; use base "Diamond_A";
 package Diamond_D; use base ("Diamond_B", "Diamond_C"); use mro "c3";
 package main; my $order = mro::get_linear_isa("Diamond_D");
               print $order->[3] eq "Diamond_A" ? "ok" : "not ok"; print "\n"'
-tests[317]='use Net::SSLeay();use IO::Socket::SSL();Net::SSLeay::OpenSSL_add_ssl_algorithms(); my $ssl_ctx = IO::Socket::SSL::SSL_Context->new(SSL_server => 1); print q(ok)'
-tests[318]='{ local $\ = "ok" ; print "" }'
+
+tests[3170]='use Net::SSLeay();use IO::Socket::SSL();Net::SSLeay::OpenSSL_add_ssl_algorithms(); my $ssl_ctx = IO::Socket::SSL::SSL_Context->new(SSL_server => 1); print q(ok)'
+tests[3180]='{ local $\ = "ok" ; print "" }'
+
+if [[ $v518 -gt 0 ]]; then
+  tests[317]='my $ok;
+  sub kt { $ok = 1 }
+  our $nested = qr/ (.) (??{ kt $1 }) /x;
+  my $re = qr/^ ( (??{ $nested }) ) $ /x;
+  "foo" =~ $re;
+  print "ok\n" if $ok'
+fi
+
 tests[319]='#TODO Wide character warnings missing (bytes layer ignored)
 use warnings q{utf8}; my $w; local $SIG{__WARN__} = sub { $w = $_[0] }; my $c = chr(300); open F, ">", "a"; binmode(F, ":bytes:"); print F $c,"\n"; close F; print $w'
-tests[320]='#TODO No warnings reading in invalid utf8 stream (utf8 layer ignored)
+if [[ $v518 -gt 0 ]]; then
+    tests[318]='use utf8; ＬＯＯＰ: { last ＬＯＯＰ } print qq(ok\n)'
+    tests[320]='use utf8; sub участники { print qq{ok\n} } $::{"участники"}->()'
+
+    tests[321]='use utf8;
+    {
+        # illegal character for one identifier
+       my $chr = "\N{POUND SIGN}";
+       # commenting this eval make the test pass
+       eval "\$$chr = 1;";
+    }
+    {
+        my $i = 0x100;
+        my $chr = chr($i);
+        eval "my \$$chr = 42;";
+        my $re = qr/^\p{_Perl_IDStart}$/;
+        print qq/ok\n/ if $chr =~ $re;
+    }'
+fi
+tests[3200]='#TODO No warnings reading in invalid utf8 stream (utf8 layer ignored)
 use warnings "utf8"; local $SIG{__WARN__} = sub { $@ = shift }; open F, ">", "a"; binmode F; my ($chrE4, $chrF6) = (chr(0xE4), chr(0xF6)); print F "foo", $chrE4, "\n"; print F "foo", $chrF6, "\n"; close F; open F, "<:utf8", "a";  undef $@; my $line = <F>; print q(ok) if $@ =~ /utf8 "\xE4" does not map to Unicode/;'
 tests[324]='package Master;
 use mro "c3";
@@ -1098,23 +1284,28 @@ print Slave->me()."\n";
 '
 result[324]='Master
 Slave of Master'
-tests[326]='#TODO method const maybe::next::method
+tests[326]='
 package Diamond_C; sub maybe { "Diamond_C::maybe" } package Diamond_D; use base "Diamond_C"; use mro "c3"; sub maybe { "Diamond_D::maybe => " . ((shift)->maybe::next::method() || 0) } package main; print "ok\n" if Diamond_D->maybe;'
 tests[328]='#WONTFIX re-eval lex/global mixup
 my $code = q[{$blah = 45}]; our $blah = 12; eval "/(?$code)/"; print "$blah\n"'
 result[328]=45
+# probably a duplicate of 295
 tests[329]='#WONTFIX re-eval lex/global mixup
 $_ = q{aaa}; my @res; pos = 1; s/\Ga(?{push @res, $_, $`})/xx/g; print "ok\n" if "$_ @res" eq "axxxx aaa a aaa aa"; print "$_ @res\n"'
 result[329]='ok
 axxxx aaa a aaa aa'
 tests[330]='"\x{101}a" =~ qr/\x{100}/i && print "ok\n"'
-tests[331]='use 5.010; use charnames ":full"; my $char = q/\N{LATIN CAPITAL LETTER A WITH MACRON}/; my $a = eval qq ["$char"]; print length($a) == 1 ? "ok\n" : "$a\n".length($a)."\n"'
+tests[331]='package Count;sub getline {print "ok\n"};BEGIN { *The::Count:: = \*Count::; };exists &The::Count::getline ? The::Count->getline() : do {eval "require Devel::Peek;"; Dump(\%The::Count::)}'
+tests[3310]='use 5.010; use charnames ":full"; my $char = q/\N{LATIN CAPITAL LETTER A WITH MACRON}/; my $a = eval qq ["$char"]; print length($a) == 1 ? "ok\n" : "$a\n".length($a)."\n"'
 tests[332]='#TODO re-eval no_modify, probably WONTFIX
 use re "eval"; our ( $x, $y, $z ) = 1..3; $x =~ qr/$x(?{ $y = $z++ })/; undef $@; print "ok\n"'
 tests[333]='use encoding "utf8";
 my @hiragana =  map {chr} ord("ぁ")..ord("ん"); my @katakana =  map {chr} ord("ァ")..ord("ン"); my $hiragana = join(q{} => @hiragana); my $katakana = join(q{} => @katakana); my %h2k; @h2k{@hiragana} = @katakana; $str = $hiragana; $str =~ s/([ぁ-ん])/$h2k{$1}/go; print $str eq $katakana ? "ok\n" : "not ok\n$hiragana\n$katakana\n";'
+tests[335]='use POSIX (); print POSIX::M_SQRT2;'
+result[335]='1.41421356237309'
 tests[338]='use utf8; my $l = "ñ"; my $re = qr/ñ/; print $l =~ $re ? qq{ok\n} : length($l)."\n".ord($l)."\n";'
 tests[340]='eval q/use Net::DNS/; my $new = "IO::Socket::INET6"->can("new") or die "die at new"; my $inet = $new->("IO::Socket::INET6", LocalAddr => q/localhost/, Proto => "udp", LocalPort => undef); print q(ok) if ref($inet) eq "IO::Socket::INET6";'
+tests[342]='use IO::Socket::INET6 (); my $sock = IO::Socket::INET6->new( Blocking => 1, PeerAddr => q/127.0.0.1/, PeerPort => 22 ); print "ok\n";'
 # used to fail in the inc-i340 branches CORE/base/lex.t 54
 tests[3401]='sub foo::::::bar { print "ok\n"; } foo::::::bar;'
 # wontfix on -O3: static string *end for "main::bar"
@@ -1122,14 +1313,79 @@ tests[345]='eval q/use Sub::Name; 1/ or die "no Sub::Name"; subname("main::bar",
 # those work fine:
 tests[3451]='eval q/use Sub::Name; 1/ or die "no Sub::Name"; subname("bar", sub { 42 } ); print "ok\n";'
 tests[3452]='eval q/use Sub::Name; 1/ or die "no Sub::Name"; $bar="main::bar"; subname($bar, sub { 42 } ); print "ok\n";'
+tests[348]='package Foo::Bar; sub baz { 1 }
+package Foo; sub new { bless {}, shift } sub method { print "ok\n"; }
+package main; Foo::Bar::baz();
+my $foo = sub {
+  Foo->new
+}->();
+$foo->method;'
+tests[350]='#TODO 5.18-5.22 dbg
+package Foo::Moose; use Moose; has bar => (is => "rw", isa => "Int");
+package main; my $moose = Foo::Moose->new; print "ok" if 32 == $moose->bar(32);'
+tests[368]='use EV; print q(ok)'
+tests[369]='
+use EV;
+use Coro;
+use Coro::Timer;
+my @a;
+push @a, async {
+  while() {
+    warn $c++;
+    Coro::Timer::sleep 1;
+  };
+};
+push @a, async {
+  while() {
+    warn $d++;
+    Coro::Timer::sleep 0.5;
+  };
+};
+schedule;
+print q(ok)'
+tests[1960]='use EV; my $w = EV::timer 1, 1,sub{print"ok\n";exit}; EV::loop'
+tests[371]='package foo;use Moose;
+has "x" => (isa => "Int", is => "rw", required => 1);
+has "y" => (isa => "Int", is => "rw", required => 1);
+sub clear { my $self = shift; $self->x(0); $self->y(0); }
+__PACKAGE__->meta->make_immutable;
+package main;
+my $f = foo->new( x => 5, y => 6);
+print $f->x . "\n";'
+result[371]='5'
+
+if [[ $v518 -gt 0 ]]; then
+  tests[372]='use utf8; require mro; my $f_gen = mro::get_pkg_gen('ᕘ'); undef %ᕘ::; mro::get_pkg_gen('ᕘ'); delete $::{"ᕘ::"}; print "ok";'
+  tests[373]='package foo; BEGIN {undef %foo::} sub doof { caller(0) } print qq/ok\n/ if +(doof())[3] =~ qr/::doof/'
+fi
+tests[2050]='use utf8;package 텟ţ::ᴼ; sub ᴼ_or_Ḋ { "ok" } print ᴼ_or_Ḋ;'
+tests[2051]='use utf8;package ƂƂƂƂ; sub ƟK { "ok" } package ƦƦƦƦ; use base "ƂƂƂƂ"; my $x = bless {}, "ƦƦƦƦ"; print $x->ƟK();'
+tests[2052]='{ package Diӑmond_A; sub fಓ { "ok" } } { package Diӑmond_B; use base q{Diӑmond_A}; use mro "c3"; sub fಓ { (shift)->next::method() } } print Diӑmond_B->fಓ();'
+# silly compiler warnings test, only usable with -q
+tests[2053]='use strict; BEGIN { $SIG{__WARN__} = sub { die "Dying on warning: ", @_ } } print q{ok}'
+# empty keys multideref
+tests[2054]='my %h; $h{""} = q/boom/; print qq{ok\n}'
+tests[2055]='our %h; $h{""} = q/boom/; print qq{ok\n}'
+# GH issues:
+tests[2790]='*TIESCALAR = sub {}; tie my $var => "main", 42; <${var}>; print qq/ok\n/'
+tests[2230]='# 5.22 SEGV with missing gv_list[0] svop_list[0]
+<*.*> and print qq{ok\n}'
+tests[3060]='INIT { $SIG{__WARN__} = sub { die } } print "ok\n";'
+tests[3061]='END { $SIG{__WARN__} = sub { die } } print "ok\n";'
+tests[2191]='sub foo1 ($\@); eval q{ foo1 "s" }; print $@ =~ /^Not enough/ ? "ok" : "";'
+tests[2192]='sub foo1 ($\%); eval q{ foo1 "s" }; print $@ =~ /^Not enough/ ? "ok" : "";'
+tests[2193]='{local $^W = 1; my $warn = "";
+local $SIG{__WARN__} = sub { $warn .= join("",@_) };
+eval q(sub badproto4 (@ $b ar) { 1; });
+print $warn =~ /Prototype after .@. for main::badproto4/ ? "ok" : $warn;}'
 
 init
 
-# 
+#
 # getopts for -q -k -E -Du,-q -v -O2, -a -c -fro-inc
-while getopts "haAckoED:B:O:f:q" opt
+while getopts "XLhaAckoED:B:O:f:q" opt
 do
-  if [ "$opt" = "q" ]; then 
+  if [ "$opt" = "q" ]; then
     QUIET=1
     CCMD="$CCMD -q"
   fi
@@ -1141,7 +1397,7 @@ do
   # -D options: u,-q for quiet, no -D for verbose, -D- for no gcc warnings
   if [ "$opt" = "D" ]; then
     OCMD="$PERL $Mblib -MO=C,-D${OPTARG},"
-    if [ $BASE = "testcc.sh" ]; then 
+    if [ $BASE = "testcc.sh" ]; then
         OCMD="$PERL $Mblib -MO=CC,-D${OPTARG},"
     fi
     if [ -z "${OPTARG/-/}" ]; then
@@ -1149,7 +1405,7 @@ do
     fi
   fi
   # -B dynamic or -B static
-  if [ "$opt" = "B" ]; then 
+  if [ "$opt" = "B" ]; then
     CCMD="$CCMD -B${OPTARG}"
   fi
   if [ "$opt" = "O" ]; then OPTIM="$OPTARG"; fi
@@ -1157,29 +1413,39 @@ do
     OCMD="$(echo $OCMD|sed -e "s/C,/C,-f$OPTARG,/")"
   fi
   if [ "$opt" = "a" ]; then # replace -Du, by -Do
-    OCMD="$(echo $OCMD|sed -r -e 's/(-D.*)u,/\1o,/')" 
+    OCMD="$(echo $OCMD|sed -r -e 's/(-D.*)u,/\1o,/')"
   fi
   if [ "$opt" = "A" ]; then
       CCMD="$CCMD -DALLOW_PERL_OPTIONS"
   fi
+  if [ "$opt" = "L" ]; then
+    make_symlinks
+    exit
+  fi
+  if [ "$opt" = "X" ]; then
+    shift
+    emit_test $1
+    exit
+  fi
 done
 
 if [ "$(perl -V:gccversion)" != "gccversion='';" ]; then
-    if [ "$(uname)" = "Darwin" ]; then
+    if [ "$(uname)" = "xxDarwin" ]; then
 	CCMD="$CCMD -g -fno-var-tracking"
     else
 	CCMD="$CCMD -g3"
     fi
 fi
+
 if [ -z $OPTIM ]; then OPTIM=-1; fi # all
 
 if [ -z "$QUIET" ]; then
-    make 
+    make
 else
     # O from 5.6 does not support -qq
     qq="`$PERL -e'print (($] < 5.007) ? q() : q(-qq,))'`"
-    # replace -D*,-v by -q 
-    OCMD="$(echo $OCMD    |sed -e 's/-D.*,//' -e 's/,-v,/,/' -e s/-MO=/-MO=$qq/)" 
+    # replace -D*,-v by -q
+    OCMD="$(echo $OCMD    |sed -e 's/-D.*,//' -e 's/,-v,/,/' -e s/-MO=/-MO=$qq/)"
     OCMDO1="$(echo $OCMDO1|sed -e 's/-D.*,//' -e 's/,-v,/,/' -e s/-MO=/-MO=$qq/)"
     OCMDO2="$(echo $OCMDO2|sed -e 's/-D.*,//' -e 's/,-v,/,/' -e s/-MO=/-MO=$qq/)"
     OCMDO3="$(echo $OCMDO3|sed -e 's/-D.*,//' -e 's/,-v,/,/' -e s/-MO=/-MO=$qq/)"
